@@ -11,10 +11,18 @@ import pytest
 
 from pandas.errors import ParserError
 
-from pandas import DataFrame, Index, MultiIndex
+from pandas import (
+    DataFrame,
+    Index,
+    MultiIndex,
+)
 import pandas._testing as tm
 
+# TODO(1.4): Change me to xfails at release time
+skip_pyarrow = pytest.mark.usefixtures("pyarrow_skip")
 
+
+@skip_pyarrow
 def test_read_with_bad_header(all_parsers):
     parser = all_parsers
     msg = r"but only \d+ lines in file"
@@ -74,7 +82,8 @@ def test_no_header_prefix(all_parsers):
 6,7,8,9,10
 11,12,13,14,15
 """
-    result = parser.read_csv(StringIO(data), prefix="Field", header=None)
+    with tm.assert_produces_warning(FutureWarning, check_stacklevel=False):
+        result = parser.read_csv(StringIO(data), prefix="Field", header=None)
     expected = DataFrame(
         [[1, 2, 3, 4, 5], [6, 7, 8, 9, 10], [11, 12, 13, 14, 15]],
         columns=["Field0", "Field1", "Field2", "Field3", "Field4"],
@@ -82,6 +91,7 @@ def test_no_header_prefix(all_parsers):
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 def test_header_with_index_col(all_parsers):
     parser = all_parsers
     data = """foo,1,2,3
@@ -119,6 +129,7 @@ baz,12,13,14,15
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 def test_header_multi_index(all_parsers):
     parser = all_parsers
     expected = tm.makeCustomDataframe(5, 3, r_idx_nlevels=2, c_idx_nlevels=4)
@@ -144,7 +155,7 @@ R_l0_g4,R_l1_g4,R4C0,R4C1,R4C2
     "kwargs,msg",
     [
         (
-            dict(index_col=["foo", "bar"]),
+            {"index_col": ["foo", "bar"]},
             (
                 "index_col must only contain "
                 "row numbers when specifying "
@@ -152,11 +163,11 @@ R_l0_g4,R_l1_g4,R4C0,R4C1,R4C2
             ),
         ),
         (
-            dict(index_col=[0, 1], names=["foo", "bar"]),
+            {"index_col": [0, 1], "names": ["foo", "bar"]},
             ("cannot specify names when specifying a multi-index header"),
         ),
         (
-            dict(index_col=[0, 1], usecols=["foo", "bar"]),
+            {"index_col": [0, 1], "usecols": ["foo", "bar"]},
             ("cannot specify usecols when specifying a multi-index header"),
         ),
     ],
@@ -181,16 +192,17 @@ R_l0_g4,R_l1_g4,R4C0,R4C1,R4C2
         parser.read_csv(StringIO(data), header=[0, 1, 2, 3], **kwargs)
 
 
-_TestTuple = namedtuple("names", ["first", "second"])
+_TestTuple = namedtuple("_TestTuple", ["first", "second"])
 
 
+@skip_pyarrow
 @pytest.mark.parametrize(
     "kwargs",
     [
-        dict(header=[0, 1]),
-        dict(
-            skiprows=3,
-            names=[
+        {"header": [0, 1]},
+        {
+            "skiprows": 3,
+            "names": [
                 ("a", "q"),
                 ("a", "r"),
                 ("a", "s"),
@@ -198,10 +210,10 @@ _TestTuple = namedtuple("names", ["first", "second"])
                 ("c", "u"),
                 ("c", "v"),
             ],
-        ),
-        dict(
-            skiprows=3,
-            names=[
+        },
+        {
+            "skiprows": 3,
+            "names": [
                 _TestTuple("a", "q"),
                 _TestTuple("a", "r"),
                 _TestTuple("a", "s"),
@@ -209,7 +221,7 @@ _TestTuple = namedtuple("names", ["first", "second"])
                 _TestTuple("c", "u"),
                 _TestTuple("c", "v"),
             ],
-        ),
+        },
     ],
 )
 def test_header_multi_index_common_format1(all_parsers, kwargs):
@@ -231,13 +243,14 @@ two,7,8,9,10,11,12"""
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 @pytest.mark.parametrize(
     "kwargs",
     [
-        dict(header=[0, 1]),
-        dict(
-            skiprows=2,
-            names=[
+        {"header": [0, 1]},
+        {
+            "skiprows": 2,
+            "names": [
                 ("a", "q"),
                 ("a", "r"),
                 ("a", "s"),
@@ -245,10 +258,10 @@ two,7,8,9,10,11,12"""
                 ("c", "u"),
                 ("c", "v"),
             ],
-        ),
-        dict(
-            skiprows=2,
-            names=[
+        },
+        {
+            "skiprows": 2,
+            "names": [
                 _TestTuple("a", "q"),
                 _TestTuple("a", "r"),
                 _TestTuple("a", "s"),
@@ -256,7 +269,7 @@ two,7,8,9,10,11,12"""
                 _TestTuple("c", "u"),
                 _TestTuple("c", "v"),
             ],
-        ),
+        },
     ],
 )
 def test_header_multi_index_common_format2(all_parsers, kwargs):
@@ -277,13 +290,14 @@ two,7,8,9,10,11,12"""
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 @pytest.mark.parametrize(
     "kwargs",
     [
-        dict(header=[0, 1]),
-        dict(
-            skiprows=2,
-            names=[
+        {"header": [0, 1]},
+        {
+            "skiprows": 2,
+            "names": [
                 ("a", "q"),
                 ("a", "r"),
                 ("a", "s"),
@@ -291,10 +305,10 @@ two,7,8,9,10,11,12"""
                 ("c", "u"),
                 ("c", "v"),
             ],
-        ),
-        dict(
-            skiprows=2,
-            names=[
+        },
+        {
+            "skiprows": 2,
+            "names": [
                 _TestTuple("a", "q"),
                 _TestTuple("a", "r"),
                 _TestTuple("a", "s"),
@@ -302,7 +316,7 @@ two,7,8,9,10,11,12"""
                 _TestTuple("c", "u"),
                 _TestTuple("c", "v"),
             ],
-        ),
+        },
     ],
 )
 def test_header_multi_index_common_format3(all_parsers, kwargs):
@@ -324,6 +338,7 @@ q,r,s,t,u,v
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 def test_header_multi_index_common_format_malformed1(all_parsers):
     parser = all_parsers
     expected = DataFrame(
@@ -344,6 +359,7 @@ q,r,s,t,u,v
     tm.assert_frame_equal(expected, result)
 
 
+@skip_pyarrow
 def test_header_multi_index_common_format_malformed2(all_parsers):
     parser = all_parsers
     expected = DataFrame(
@@ -365,6 +381,7 @@ q,r,s,t,u,v
     tm.assert_frame_equal(expected, result)
 
 
+@skip_pyarrow
 def test_header_multi_index_common_format_malformed3(all_parsers):
     parser = all_parsers
     expected = DataFrame(
@@ -385,6 +402,19 @@ q,r,s,t,u,v
     tm.assert_frame_equal(expected, result)
 
 
+@skip_pyarrow
+def test_header_multi_index_blank_line(all_parsers):
+    # GH 40442
+    parser = all_parsers
+    data = [[None, None], [1, 2], [3, 4]]
+    columns = MultiIndex.from_tuples([("a", "A"), ("b", "B")])
+    expected = DataFrame(data, columns=columns)
+    data = "a,b\nA,B\n,\n1,2\n3,4"
+    result = parser.read_csv(StringIO(data), header=[0, 1])
+    tm.assert_frame_equal(expected, result)
+
+
+@skip_pyarrow
 @pytest.mark.parametrize(
     "data,header", [("1,2,3\n4,5,6", None), ("foo,bar,baz\n1,2,3\n4,5,6", 0)]
 )
@@ -397,7 +427,8 @@ def test_header_names_backward_compat(all_parsers, data, header):
     tm.assert_frame_equal(result, expected)
 
 
-@pytest.mark.parametrize("kwargs", [dict(), dict(index_col=False)])
+@skip_pyarrow
+@pytest.mark.parametrize("kwargs", [{}, {"index_col": False}])
 def test_read_only_header_no_rows(all_parsers, kwargs):
     # See gh-7773
     parser = all_parsers
@@ -410,10 +441,10 @@ def test_read_only_header_no_rows(all_parsers, kwargs):
 @pytest.mark.parametrize(
     "kwargs,names",
     [
-        (dict(), [0, 1, 2, 3, 4]),
-        (dict(prefix="X"), ["X0", "X1", "X2", "X3", "X4"]),
+        ({}, [0, 1, 2, 3, 4]),
+        ({"prefix": "X"}, ["X0", "X1", "X2", "X3", "X4"]),
         (
-            dict(names=["foo", "bar", "baz", "quux", "panda"]),
+            {"names": ["foo", "bar", "baz", "quux", "panda"]},
             ["foo", "bar", "baz", "quux", "panda"],
         ),
     ],
@@ -427,7 +458,11 @@ def test_no_header(all_parsers, kwargs, names):
     expected = DataFrame(
         [[1, 2, 3, 4, 5], [6, 7, 8, 9, 10], [11, 12, 13, 14, 15]], columns=names
     )
-    result = parser.read_csv(StringIO(data), header=None, **kwargs)
+    if "prefix" in kwargs.keys():
+        with tm.assert_produces_warning(FutureWarning, check_stacklevel=False):
+            result = parser.read_csv(StringIO(data), header=None, **kwargs)
+    else:
+        result = parser.read_csv(StringIO(data), header=None, **kwargs)
     tm.assert_frame_equal(result, expected)
 
 
@@ -442,6 +477,7 @@ def test_non_int_header(all_parsers, header):
         parser.read_csv(StringIO(data), header=header)
 
 
+@skip_pyarrow
 def test_singleton_header(all_parsers):
     # see gh-7757
     data = """a,b,c\n0,1,2\n1,2,3"""
@@ -452,6 +488,7 @@ def test_singleton_header(all_parsers):
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 @pytest.mark.parametrize(
     "data,expected",
     [
@@ -498,6 +535,7 @@ def test_mangles_multi_index(all_parsers, data, expected):
     tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
 @pytest.mark.parametrize("index_col", [None, [0]])
 @pytest.mark.parametrize(
     "columns", [None, (["", "Unnamed"]), (["Unnamed", ""]), (["Unnamed", "NotUnnamed"])]
@@ -519,28 +557,37 @@ def test_multi_index_unnamed(all_parsers, index_col, columns):
     else:
         data = ",".join([""] + (columns or ["", ""])) + "\n,0,1\n0,2,3\n1,4,5\n"
 
+    result = parser.read_csv(StringIO(data), header=header, index_col=index_col)
+    exp_columns = []
+
     if columns is None:
-        msg = (
-            r"Passed header=\[0,1\] are too "
-            r"many rows for this multi_index of columns"
-        )
-        with pytest.raises(ParserError, match=msg):
-            parser.read_csv(StringIO(data), header=header, index_col=index_col)
-    else:
-        result = parser.read_csv(StringIO(data), header=header, index_col=index_col)
-        exp_columns = []
+        columns = ["", "", ""]
 
-        for i, col in enumerate(columns):
-            if not col:  # Unnamed.
-                col = f"Unnamed: {i if index_col is None else i + 1}_level_0"
+    for i, col in enumerate(columns):
+        if not col:  # Unnamed.
+            col = f"Unnamed: {i if index_col is None else i + 1}_level_0"
 
-            exp_columns.append(col)
+        exp_columns.append(col)
 
-        columns = MultiIndex.from_tuples(zip(exp_columns, ["0", "1"]))
-        expected = DataFrame([[2, 3], [4, 5]], columns=columns)
-        tm.assert_frame_equal(result, expected)
+    columns = MultiIndex.from_tuples(zip(exp_columns, ["0", "1"]))
+    expected = DataFrame([[2, 3], [4, 5]], columns=columns)
+    tm.assert_frame_equal(result, expected)
 
 
+@skip_pyarrow
+def test_names_longer_than_header_but_equal_with_data_rows(all_parsers):
+    # GH#38453
+    parser = all_parsers
+    data = """a, b
+1,2,3
+5,6,4
+"""
+    result = parser.read_csv(StringIO(data), header=0, names=["A", "B", "C"])
+    expected = DataFrame({"A": [1, 5], "B": [2, 6], "C": [3, 4]})
+    tm.assert_frame_equal(result, expected)
+
+
+@skip_pyarrow
 def test_read_csv_multiindex_columns(all_parsers):
     # GH#6051
     parser = all_parsers
@@ -570,3 +617,64 @@ def test_read_csv_multiindex_columns(all_parsers):
     tm.assert_frame_equal(df1, expected.iloc[:1])
     df2 = parser.read_csv(StringIO(s2), header=[0, 1])
     tm.assert_frame_equal(df2, expected)
+
+
+@skip_pyarrow
+def test_read_csv_multi_header_length_check(all_parsers):
+    # GH#43102
+    parser = all_parsers
+
+    case = """row11,row12,row13
+row21,row22, row23
+row31,row32
+"""
+
+    with pytest.raises(
+        ParserError, match="Header rows must have an equal number of columns."
+    ):
+        parser.read_csv(StringIO(case), header=[0, 2])
+
+
+@skip_pyarrow
+def test_header_none_and_implicit_index(all_parsers):
+    # GH#22144
+    parser = all_parsers
+    data = "x,1,5\ny,2\nz,3\n"
+    result = parser.read_csv(StringIO(data), names=["a", "b"], header=None)
+    expected = DataFrame(
+        {"a": [1, 2, 3], "b": [5, np.nan, np.nan]}, index=["x", "y", "z"]
+    )
+    tm.assert_frame_equal(result, expected)
+
+
+@skip_pyarrow
+def test_header_none_and_implicit_index_in_second_row(all_parsers):
+    # GH#22144
+    parser = all_parsers
+    data = "x,1\ny,2,5\nz,3\n"
+    with pytest.raises(ParserError, match="Expected 2 fields in line 2, saw 3"):
+        parser.read_csv(StringIO(data), names=["a", "b"], header=None)
+
+
+@skip_pyarrow
+def test_header_none_and_on_bad_lines_skip(all_parsers):
+    # GH#22144
+    parser = all_parsers
+    data = "x,1\ny,2,5\nz,3\n"
+    result = parser.read_csv(
+        StringIO(data), names=["a", "b"], header=None, on_bad_lines="skip"
+    )
+    expected = DataFrame({"a": ["x", "z"], "b": [1, 3]})
+    tm.assert_frame_equal(result, expected)
+
+
+@skip_pyarrow
+def test_header_missing_rows(all_parsers):
+    # GH#47400
+    parser = all_parsers
+    data = """a,b
+1,2
+"""
+    msg = r"Passed header=\[0,1,2\], len of 3, but only 2 lines in file"
+    with pytest.raises(ValueError, match=msg):
+        parser.read_csv(StringIO(data), header=[0, 1, 2])
