@@ -4,6 +4,7 @@ import pytest
 import pytz  # noqa  # a test below uses pytz but only inside a `eval` call
 
 from pandas import Timestamp
+import pandas._testing as tm
 
 
 class TestTimestampRendering:
@@ -17,7 +18,7 @@ class TestTimestampRendering:
     )
     def test_repr(self, date, freq, tz):
         # avoid to match with timezone name
-        freq_repr = "'{0}'".format(freq)
+        freq_repr = f"'{freq}'"
         if tz.startswith("dateutil"):
             tz_repr = tz.replace("dateutil", "")
         else:
@@ -35,17 +36,26 @@ class TestTimestampRendering:
         assert freq_repr not in repr(date_tz)
         assert date_tz == eval(repr(date_tz))
 
-        date_freq = Timestamp(date, freq=freq)
+        msg = "The 'freq' argument in Timestamp"
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            date_freq = Timestamp(date, freq=freq)
         assert date in repr(date_freq)
         assert tz_repr not in repr(date_freq)
         assert freq_repr in repr(date_freq)
-        assert date_freq == eval(repr(date_freq))
+        with tm.assert_produces_warning(
+            FutureWarning, match=msg, check_stacklevel=False
+        ):
+            assert date_freq == eval(repr(date_freq))
 
-        date_tz_freq = Timestamp(date, tz=tz, freq=freq)
+        with tm.assert_produces_warning(FutureWarning, match=msg):
+            date_tz_freq = Timestamp(date, tz=tz, freq=freq)
         assert date in repr(date_tz_freq)
         assert tz_repr in repr(date_tz_freq)
         assert freq_repr in repr(date_tz_freq)
-        assert date_tz_freq == eval(repr(date_tz_freq))
+        with tm.assert_produces_warning(
+            FutureWarning, match=msg, check_stacklevel=False
+        ):
+            assert date_tz_freq == eval(repr(date_tz_freq))
 
     def test_repr_utcoffset(self):
         # This can cause the tz field to be populated, but it's redundant to
@@ -85,3 +95,13 @@ class TestTimestampRendering:
          {'w': {'a': Timestamp('2011-01-01 00:00:00')}}],
  'foo': 1}"""
         assert result == expected
+
+    def test_to_timestamp_repr_is_code(self):
+        zs = [
+            Timestamp("99-04-17 00:00:00", tz="UTC"),
+            Timestamp("2001-04-17 00:00:00", tz="UTC"),
+            Timestamp("2001-04-17 00:00:00", tz="America/Los_Angeles"),
+            Timestamp("2001-04-17 00:00:00", tz=None),
+        ]
+        for z in zs:
+            assert eval(repr(z)) == z
