@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 import sys
 import types
 
@@ -16,21 +15,16 @@ def test_safe_import_exists():
     assert td.safe_import("pandas")
 
 
-@pytest.mark.parametrize("min_version,valid", [
-    ("0.0.0", True),
-    ("99.99.99", False)
-])
+@pytest.mark.parametrize("min_version,valid", [("0.0.0", True), ("99.99.99", False)])
 def test_safe_import_versions(min_version, valid):
     result = td.safe_import("pandas", min_version=min_version)
     result = result if valid else not result
     assert result
 
 
-@pytest.mark.parametrize("min_version,valid", [
-    (None, False),
-    ("1.0", True),
-    ("2.0", False)
-])
+@pytest.mark.parametrize(
+    "min_version,valid", [(None, False), ("1.0", True), ("2.0", False)]
+)
 def test_safe_import_dummy(monkeypatch, min_version, valid):
     mod_name = "hello123"
 
